<?php
/**
 * Created by VisualCode.
 * User: WellingtonLima
 * Date: 29/06/2021
 * Time: 11:59
 */

namespace Schaffen\ConciliacaoBancaria\Cnab\Conciliacao;

use Schaffen\ConciliacaoBancaria\Contracts\Cnab\Conciliacao\Cnab240\Header as Header240Contract;
use Schaffen\ConciliacaoBancaria\Contracts\Cnab\Conciliacao\Cnab240\Trailer as Trailer240Contract;
use Schaffen\ConciliacaoBancaria\Util;

abstract class AbstractRetorno 
{
    /**
     * Se Cnab ja foi processado
     *
     * @var bool
     */
    protected $processado = false;

    /**
     * Código do banco
     *
     * @var string
     */
    protected $codigoBanco;

   
    /**
     * Arquivo transformado em array por linha.
     *
     * @var array
     */
    protected $file;

    /**
     * @var Header240Contract
     */
    public $header;

    /**
     * @var Trailer240Contract
     */
    public $trailer;

     /**
     * Incremeto de detalhes
     *
     * @var int
     */
    protected $increment = 0;

     /**
     * Incremeto de lote
     *
     * @var int
     */
    protected $incrementLote = 0;

     /**
     * Incremeto de lote
     *
     * @var int
     */
    protected $countLinha = 0;


    /**
     * @var array
     */
    public $lotes = [];

    /**
     * @var array
     */
    public $detalhes_array = [];

    /**
     * Helper de totais.
     *
     * @var array
     */
    protected $totais = [];

    /**
     *
     * @param String $file
     * @throws \Exception
     */
    public function __construct($file)
    {
        $this->_position = 1;

        if (!$this->file = Util::file2array($file)) {
            throw new \Exception("Arquivo: não existe");
        }

        $r = new \ReflectionClass('\Schaffen\ConciliacaoBancaria\Contracts\Boleto\Boleto');
        $constantNames = $r->getConstants();
        $bancosDisponiveis = [];
        foreach ($constantNames as $constantName => $codigoBanco) {
            if (preg_match('/^COD_BANCO.*/', $constantName)) {
                $bancosDisponiveis[] = $codigoBanco;
            }
        }

        if (!Util::isHeaderRetorno($this->file[0])) {
            throw new \Exception(sprintf("Arquivo de retorno inválido"));
        }

        $banco = Util::isCnab400($this->file[0]) ? mb_substr($this->file[0], 76, 3) : mb_substr($this->file[0], 0, 3);
        if (!in_array($banco, $bancosDisponiveis)) {
            throw new \Exception(sprintf("Banco: %s, inválido", $banco));
        }
    }

    /**
     * Retorna o código do banco
     *
     * @return string
     */
    public function getCodigoBanco()
    {
        return $this->codigoBanco;
    }

    /**
     * @return mixed
     */
    public function getBancoNome()
    {
        return Util::$bancos[$this->codigoBanco];
    }

    /**
     * @return mixed
     */
    public function getTipo()
    {
        return Util::isCnab400($this->file[0]) ? 400 : 240;
    }

    /**
     * @return mixed
     */
    public function getFileContent()
    {
        return implode(PHP_EOL, $this->file);
    }

    /**
     * @return Header240Contract
     */
    public function getHeader()
    {
        return $this->header;
    }

    /**
     * @return Trailer240Contract
     */
    public function getTrailer()
    {
        return $this->trailer;
    }

    /**
     * @return array
     */
    public function getTotais()
    {
        return $this->totais;
    }

    /**
     * Se esta processado
     *
     * @return bool
     */
    protected function isProcessado()
    {
        return $this->processado;
    }
    /**
     * Seta cnab como processado
     *
     * @return $this
     */
    protected function setProcessado()
    {
        $this->processado = true;
        return $this;
    }

    /**
     * Incrementa o lote.
     * @param $atributo
     */
    abstract protected function incrementLote($atributo);

    /**
     * Processa o arquivo
     *
     * @return $this
     */
    abstract protected function processar();

    /**
     * Retorna o array.
     *
     * @return array
     */
    abstract protected function toArray();

    /**
     * Remove trecho do array.
     *
     * @param $i
     * @param $f
     * @param $array
     *
     * @return string
     * @throws \Exception
     */
    protected function rem($i, $f, &$array)
    {
        return Util::remove($i, $f, $array);
    }

}
