<?php
namespace Schaffen\ConciliacaoBancaria\Cnab\Conciliacao\Cnab240;

use \Schaffen\ConciliacaoBancaria\Cnab\Conciliacao\AbstractRetorno as AbstractRetornoGeneric;
use Schaffen\ConciliacaoBancaria\Contracts\Cnab\Conciliacao\Cnab240\HeaderLote as HeaderLoteContract;
use Schaffen\ConciliacaoBancaria\Contracts\Cnab\Conciliacao\Cnab240\TrailerLote as TrailerLoteContract;
use Schaffen\ConciliacaoBancaria\Contracts\Cnab\Conciliacao\Cnab240\Detalhe as DetalheContract;
use Illuminate\Support\Collection;

/**
 * Class AbstractRetorno
 *
 * @method  \Schaffen\ConciliacaoBancaria\Cnab\Conciliacao\Cnab240\Header getHeader()
 * @method  \Schaffen\ConciliacaoBancaria\Cnab\Conciliacao\Cnab240\Trailer getTrailer()
 * @package Schaffen\ConciliacaoBancaria\Cnab\Conciliacao\Cnab240
 */
abstract class AbstractRetorno extends AbstractRetornoGeneric
{
    /**
     * @var HeaderLoteContract
     */
    public $headerLote;

    /**
     * @var TrailerLoteContract
     */
    public $trailerLote;

    /**
     * @var DetalheContract
     */
    public $detalhe;

    

    /**
     * @param String $file
     * @throws \Exception
     */
    public function __construct($file)
    {
        parent::__construct($file);

        $this->header = new Header();
        $this->trailer = new Trailer();
    }

    /**
     * @return HeaderLote
     */
    public function getHeaderLote()
    {
        return $this->headerLote;
    }

    /**
     * @return Detalhe
     */
    public function getDetalheConciliacao()
    {
        return $this->detalhe;
    }

    /**
     * @return TrailerLote
     */
    public function getTrailerLote()
    {
        return $this->trailerLote;
    }

    /**
     * @param array $header
     *
     * @return boolean
     */
    abstract protected function processarHeader(array $header);

    /**
     * @param array $headerLote
     *
     * @return boolean
     */
    abstract protected function processarHeaderLote(array $headerLote);

    /**
     * @param array $detalhe
     *
     * @return boolean
     */
    abstract protected function processarDetalhe(array $detalhe);

    /**
     * @param array $trailerLote
     *
     * @return boolean
     */
    abstract protected function processarTrailerLote(array $trailerLote);

    /**
     * @param array $trailer
     *
     * @return boolean
     */
    abstract protected function processarTrailer(array $trailer);

    /**
     * Incrementa o Lote.
     */
    protected function incrementLote($atributo)
    {
        $this->lotes[$this->incrementLote] = $atributo;
    }




    /**
     * Processa o Lote
     *
     * @return $this
     * @throws \Exception
     */
    public function processarLote($linha, $tipo){

        if ($tipo == '1') {
            $this->headerLote = new HeaderLote();
            $this->incrementLote++;
            $this->detalhes_array = array();
            $this->processarHeaderLote($linha);
            
            
        } elseif ($tipo == '3') {
            $this->detalhe = new Detalhe();
            array_push($this->detalhes_array, $this->detalhe);
            $this->processarDetalhe($linha);

        } elseif ($tipo == '5') {
            $this->trailerLote = new TrailerLote();
            $aux = [
                'HeaderLote' => $this->headerLote,
                'Detalhe' => $this->detalhes_array,
                'TrailerLote' => $this->trailerLote,
            ];
            $this->processarTrailerLote($linha);
            $this->incrementLote($aux);
            
            
        }
    }


    /**
     * Processa o arquivo
     *
     * @return $this
     * @throws \Exception
     */
    public function processar()
    {
        if ($this->isProcessado()) {
            return $this;
        }
        foreach ($this->file as $linha) {
            $this->countLinha++;
            $recordType = $this->rem(8, 8, $linha);
            if ($recordType == '0') {
                $this->processarHeader($linha);
            } elseif ($recordType == '9') {
                $this->processarTrailer($linha);
            } else { // Lote
                $this->processarLote($linha, $recordType);
            } 
        }
        
        return $this->setProcessado();
    }

    /**
     * Retorna o array.
     *
     * @return array
     */
    public function toArray()
    {
        $array = [
            'header' => $this->header,
            'lote' => [
                'headerLote' => $this->headerLote,
                'trailerLote' => $this->trailerLote,
                'detalhes' => $this->detalhes_array,
            ],
            'trailer' => $this->trailer,
            
        ];

        return $array;

    }

    protected function getSegmentType($line)
    {
        return strtoupper($this->rem(14, 14, $line));
    }
}
